@@ -12,12 +12,14 @@ import asyncio
 import binascii
 import collections
 import logging
+import sys
 import time
+import warnings
 
 from typing import Callable, Dict, List, Optional, Union
 
 from jaeger import can_log, config, log, maskbits
-from jaeger.exceptions import CommandError, JaegerError
+from jaeger.exceptions import CommandError, JaegerError, JaegerUserWarning
 from jaeger.interfaces import BusABC, Message
 from jaeger.maskbits import CommandStatus, ResponseCode
 from jaeger.utils import StatusMixIn, get_identifier, parse_identifier
@@ -163,7 +165,13 @@ class EmptyPool(CommandError):
 data_co = Union[None, bytearray, List[bytearray]]
 
 
-class Command(StatusMixIn[CommandStatus], asyncio.Future):
+if sys.version_info >= (3, 9):
+    Future_co = asyncio.Future["Command"]
+else:
+    Future_co = asyncio.Future
+
+
+class Command(StatusMixIn[CommandStatus], Future_co):
     """A command to be sent to the CAN controller.
 
     Implements a base class to define CAN commands to interact with the
@@ -414,7 +422,7 @@ class Command(StatusMixIn[CommandStatus], asyncio.Future):
 
         return True
 
-    def process_reply(self, reply_message):
+    async def process_reply(self, reply_message):
         """Watches the reply queue."""
 
         reply = Reply(reply_message, command=self)
@@ -460,6 +468,13 @@ class Command(StatusMixIn[CommandStatus], asyncio.Future):
             f"data={data_hex!r}"
         )
 
+        if reply.response_code != ResponseCode.COMMAND_ACCEPTED:
+            warnings.warn(
+                f"Positioner {reply.positioner_id} replied to {self.name} "
+                f"UID={self.command_uid} with {reply.response_code.name!r}.",
+                JaegerUserWarning,
+            )
+
         reply_status = self._check_replies()
         if reply_status is True:
             reply_codes = [reply.response_code for reply in self.replies]
@@ -562,15 +577,16 @@ class Command(StatusMixIn[CommandStatus], asyncio.Future):
 
             pid_data = self.data[pid]
 
-            try:
-                uid = UID_POOL[cid][pid].pop()
-            except KeyError:
-                # Before failing, put back the UIDs of the other messages
-                for message in messages:
-                    UID_POOL[cid][pid].add(message.uid)
-                raise EmptyPool("no UIDs left in the pool.")
-
             for d in pid_data:
+
+                try:
+                    uid = UID_POOL[cid][pid].pop()
+                except KeyError:
+                    # Before failing, put back the UIDs of the other messages
+                    for message in messages:
+                        UID_POOL[cid][pid].add(message.uid)
+                    raise EmptyPool("no UIDs left in the pool.")
+
                 messages.append(SuperMessage(self, positioner_id=pid, uid=uid, data=d))
 
         return messages
