@@ -6,6 +6,8 @@
 # @Filename: bootloader.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
+from __future__ import annotations
+
 import asyncio
 import contextlib
 import os
@@ -14,15 +16,21 @@ import time
 import warnings
 import zlib
 
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional
+
 import numpy
 
-from jaeger import log
+from jaeger import can_log, config, log
 from jaeger.commands import Command, CommandID
 from jaeger.exceptions import JaegerError, JaegerUserWarning
 from jaeger.maskbits import BootloaderStatus
 from jaeger.utils import int_to_bytes
 
 
+if TYPE_CHECKING:
+    from jaeger import FPS
+
+
 __all__ = [
     "load_firmware",
     "StartFirmwareUpgrade",
@@ -32,12 +40,14 @@ __all__ = [
 
 
 async def load_firmware(
-    fps,
-    firmware_file,
-    positioners=None,
-    force=False,
-    show_progressbar=False,
-    progress_callback=None,
+    fps: FPS,
+    firmware_file: str | pathlib.Path,
+    positioners: Optional[List[int]] = None,
+    messages_per_positioner: Optional[int] = None,
+    force: bool = False,
+    show_progressbar: bool = False,
+    progress_callback: Optional[Callable[[int, int], Any]] = None,
+    stop_logging: bool = True,
 ):
     """Convenience function to run through the steps of loading a new firmware.
 
@@ -46,23 +56,30 @@ async def load_firmware(
 
     Parameters
     ----------
-    fps : `~jaeger.fps.FPS`
+    fps
         `~jaeger.fps.FPS` instance to which the commands will be sent.
-    firmware_file : str
+    firmware_file
         Binary file containing the firmware to load.
-    positioners : `list` or `None`
+    positioners
         A list of positioner ids whose firmware to update, or `None` to update
         all the positioners in ``fps``.
-    force : bool
+    messages_per_positioner
+        How many messages to send to each positioner at once. This can improve the
+        performance but also overflow the CAN bus buffer. With the default value of
+        `None`, reverts to the configuration value
+        ``positioner.firmware_messages_per_positioner``.
+    force
         Forces the firmware load to continue even if some positioners are not
         responding or are not in bootloader mode.
-    show_progressbar : bool
+    show_progressbar
         Whether to show a progress bar.
-    progress_callback : bool
+    progress_callback
         A function to call as data gets transferred to the positioners. The
         callback is called with ``(current_chunk, n_chuck)`` where
         ``current_chunk`` is the number of the data chunk being sent and
         ``n_chunk`` is the total number of chunks in the data package.
+    stop_logging
+        Disable logging to file for the CAN logger to improve performance.
 
     """
 
@@ -117,10 +134,8 @@ async def load_firmware(
         raise JaegerError(
             "no positioners found in bootloader mode or with valid status."
         )
-        return
 
     if n_bad > 0:
-
         msg = f"{n_bad} positioners not in bootloader mode or state is invalid."
         if force:
             warnings.warn(msg + " Proceeding becasuse force=True.", JaegerUserWarning)
@@ -137,18 +152,15 @@ async def load_firmware(
     log.info(f"CRC32: {crc32}")
     log.info(f"File size: {filesize} bytes")
 
-    cmds = [
-        fps.send_command(
-            CommandID.START_FIRMWARE_UPGRADE,
-            positioner_id=positioner.positioner_id,
-            data=start_firmware_payload,
-        )
-        for positioner in valid_positioners
-    ]
+    pids = [pos.positioner_id for pos in valid_positioners]
 
-    await asyncio.gather(*cmds)
+    cmd = await fps.send_command(
+        CommandID.START_FIRMWARE_UPGRADE,
+        positioner_ids=pids,
+        data=[start_firmware_payload],
+    )
 
-    if any(cmd.status.failed or cmd.status.timed_out for cmd in cmds):
+    if cmd.status.failed or cmd.status.timed_out:
         log.error("firmware upgrade failed.")
         return False
 
@@ -157,51 +169,72 @@ async def load_firmware(
 
     log.info("starting data send.")
 
+    if stop_logging and can_log.fh:
+        fh_handler = can_log.handlers.pop(can_log.handlers.index(can_log.fh))
+    else:
+        fh_handler = None
+
     chunk_size = 8
     n_chunks = int(numpy.ceil(filesize / chunk_size))
 
     with contextlib.ExitStack() as stack:
 
-        if show_progressbar:
+        if show_progressbar and progressbar:
             bar = stack.enter_context(progressbar.ProgressBar(max_value=n_chunks))
         else:
             bar = None
 
+        messages_default = config["positioner"]["firmware_messages_per_positioner"]
+        messages_per_positioner = messages_per_positioner or messages_default
+        assert isinstance(messages_per_positioner, int)
+
         ii = 0
         while True:
+            cmds = []
+            stop = False
+            for __ in range(messages_per_positioner):
+                chunk = firmware_data.read(chunk_size)
+                packetdata = bytearray(chunk)
+                # packetdata.reverse()  # IMPORTANT! no longer needed for P1
 
-            chunk = firmware_data.read(chunk_size)
-            packetdata = bytearray(chunk)
-            # packetdata.reverse()  # IMPORTANT! no longer needed for P1
+                if len(packetdata) == 0:
+                    stop = True
+                    break
 
-            if len(packetdata) == 0:
-                break
-
-            cmds = [
-                fps.send_command(
-                    CommandID.SEND_FIRMWARE_DATA,
-                    positioner_id=positioner.positioner_id,
-                    data=packetdata,
-                    timeout=15,
+                cmds.append(
+                    fps.send_command(
+                        CommandID.SEND_FIRMWARE_DATA,
+                        positioner_ids=pids,
+                        data=[packetdata],
+                        timeout=15,
+                    )
                 )
-                for positioner in valid_positioners
-            ]
 
             await asyncio.gather(*cmds)
 
             if any(cmd.status.failed or cmd.status.timed_out for cmd in cmds):
                 log.error("firmware upgrade failed.")
+                if fh_handler:
+                    can_log.addHandler(fh_handler)
                 return False
 
-            ii += 1
-            if show_progressbar:
-                bar.update(ii)
+            ii += messages_per_positioner
+
+            if show_progressbar and bar:
+                if ii < n_chunks:
+                    bar.update(ii)
 
             if progress_callback:
                 progress_callback(ii, n_chunks)
 
+            if stop:
+                break
+
     log.info("firmware upgrade complete.")
 
+    if fh_handler:
+        can_log.addHandler(fh_handler)
+
     total_time = time.time() - start_time
     log.info(f"upgrading firmware took {total_time:.2f}")
 
@@ -215,23 +248,22 @@ class GetFirmwareVersion(Command):
     safe = True
     bootloader = True
 
-    def get_firmware(self, positioner_id=None):
+    def get_firmware(self, positioner_id=None) -> str | Dict[int, str] | None:
         """Returns the firmware version string.
 
         Parameters
         ----------
         positioner_id : int
-            The positioner for which to return the version. This parameter is
-            ignored unless the command is a broadcast. If `None` and the
-            command is a broadcast, returns a list with the firmware version of
-            all the positioners, in the order of `GetFirmwareVersion.replies`.
+            The positioner for which to return the version. If `None` returns
+            a dictionary with the firmware version of all the positioners that
+            replied.
 
         Returns
         -------
-        firmware : `str` or `list`
-            A string or list of string with the firmware version(s), with the
-            format ``'XX.YY.ZZ'`` where ``YY='80'`` if the positioner is in
-            bootloader mode.
+        firmware
+            A string or dictionary of string with the firmware version(s),
+            with the format ``'XX.YY.ZZ'`` where ``YY='80'`` if the positioner
+            is in bootloader mode.
 
         Raises
         ------
@@ -243,21 +275,21 @@ class GetFirmwareVersion(Command):
         def format_version(reply):
             return ".".join(format(byt, "02d") for byt in reply.data[0:3][::-1])
 
-        # If not a broadcast, use the positioner_id of the command
-        if self.positioner_id != 0:
-            positioner_id = self.positioner_id
-
         if len(self.replies) == 0:
-            raise ValueError("no positioners have replied to this command.")
+            raise ValueError("No positioners have replied to this command.")
 
-        if positioner_id is None:
-            return [format_version(reply) for reply in self.replies]
-        else:
-            reply = self.get_reply_for_positioner(positioner_id)
-            if reply:
-                return format_version(reply)
+        firmwares = {}
+        for reply in self.replies:
+            version = format_version(reply)
+            if positioner_id is not None and reply.positioner_id == positioner_id:
+                return version
             else:
-                return None
+                firmwares[reply.positioner_id] = version
+
+        if positioner_id is not None:
+            return None
+
+        return firmwares
 
     @staticmethod
     def encode(firmware):
