@@ -345,3 +345,19 @@ class PreciseMoveBetaOff(Command):
     broadcastable = False
     move_command = False
     safe = False
+
+
+class SetIncreaseCollisionMargin(Command):
+    """Sets the buffer for collision margin."""
+
+    command_id = CommandID.SET_INCREASE_COLLISION_MARGIN
+    broadcastable = False
+    move_command = False
+    safe = False
+
+    def __init__(self, positioner_ids, buffer: int, **kwargs):
+
+        data = int_to_bytes(int(buffer))
+        kwargs["data"] = data
+
+        super().__init__(positioner_ids, **kwargs)
