@@ -111,7 +111,7 @@ async def send_trajectory(
     if traj.failed:
         raise TrajectoryError("Something went wrong sending the trajectory.")
 
-    log.info(f"Trajectory successfully sent in {traj.data_send_time:1f} seconds.")
+    log.debug(f"Trajectory successfully sent in {traj.data_send_time:1f} seconds.")
     log.info(f"Expected time to complete trajectory: {traj.move_time:.2f} seconds.")
 
     log.info("starting trajectory ...")
@@ -154,7 +154,7 @@ class Trajectory(object):
     Given the following two-point trajectory for positioner 4 ::
 
         points = {4: {'alpha': [(90, 0), (91, 3)],
-                    'beta': [(20, 0), (23, 4)]}}
+                      'beta': [(20, 0), (23, 4)]}}
 
     the normal process to execute the trajectory is ::
 
@@ -174,7 +174,9 @@ class Trajectory(object):
         self.trajectories: TrajectoryDataType
 
         if self.fps.locked:
-            raise FPSLockedError("FPS is locked. Cannot send trajectories.")
+            raise FPSLockedError(
+                f"FPS is locked by {fps.locked_by}. Cannot send trajectories."
+            )
 
         if self.fps.moving:
             raise TrajectoryError("The FPS is moving. Cannot send new trajectory.")
@@ -207,7 +209,7 @@ class Trajectory(object):
         if len(self.trajectories) == 0:
             raise TrajectoryError("trajectory is empty.")
 
-        if len(self.trajectories) != len(numpy.unique(list(self.trajectories.keys()))):
+        if len(self.trajectories) != len(numpy.unique(list(self.trajectories))):
             raise TrajectoryError("Duplicate positioner trajectories.")
 
         for pid in self.trajectories:
@@ -219,8 +221,8 @@ class Trajectory(object):
             for arm in ["alpha", "beta"]:
                 data = numpy.array(list(zip(*trajectory[arm]))[0])
 
-                if numpy.any(data > 360) or numpy.any(data < 0):
-                    raise TrajectoryError(f"Positioner {pid} has points out of range.")
+                # if numpy.any(data > 360) or numpy.any(data < 0):
+                #     raise TrajectoryError(f"Positioner {pid} has points out of range.")
 
                 if arm == "beta":
                     if config.get("safe_mode", False):
@@ -230,8 +232,8 @@ class Trajectory(object):
                             min_beta = config["safe_mode"]["min_beta"]
                         if numpy.any(data < min_beta):
                             raise TrajectoryError(
-                                f"Oositioner {pid}: safe mode is "
-                                f"on and beta < {min_beta}."
+                                f"Positioner {pid}: safe mode is on "
+                                f"and beta < {min_beta}."
                             )
 
     async def send(self):
@@ -242,7 +244,7 @@ class Trajectory(object):
         await self.fps.stop_trajectory()
 
         if not await self.fps.update_status(
-            positioner_ids=list(self.trajectories.keys()),
+            positioner_ids=list(self.trajectories),
             timeout=1.0,
         ):
             self.failed = True
@@ -287,18 +289,23 @@ class Trajectory(object):
             if max_time_pos > self.move_time:
                 self.move_time = max_time_pos
 
-        # Starts trajectory
-        new_traj_cmds = [
-            self.fps.send_command(
-                "SEND_NEW_TRAJECTORY",
-                positioner_ids=pos_id,
-                n_alpha=self.n_points[pos_id][0],
-                n_beta=self.n_points[pos_id][1],
+        new_traj_data = {}
+        for pos_id in self.trajectories:
+            data = SendNewTrajectory.get_data(
+                self.n_points[pos_id][0],
+                self.n_points[pos_id][1],
             )
-            for pos_id in self.trajectories
-        ]
+            new_traj_data[pos_id] = data
 
-        await asyncio.gather(*new_traj_cmds)
+        # Starts trajectory
+        new_traj_cmd = await self.fps.send_command(
+            "SEND_NEW_TRAJECTORY",
+            positioner_ids=list(self.trajectories),
+            data=new_traj_data,
+        )
+
+        if new_traj_cmd.status.failed or new_traj_cmd.status.timed_out:
+            raise TrajectoryError("Failed sending SEND_NEW_TRAJECTORY.")
 
         start_trajectory_send_time = time.time()
 
@@ -315,7 +322,8 @@ class Trajectory(object):
 
             for jj in range(0, max_points[arm], n_chunk):
 
-                data_cmds = []
+                data = {}
+                send_trajectory_pids = []
 
                 for pos_id in self.trajectories:
 
@@ -323,23 +331,25 @@ class Trajectory(object):
                     if len(arm_chunk) == 0:
                         continue
 
-                    data_cmds.append(
-                        self.fps.send_command(
-                            "SEND_TRAJECTORY_DATA",
-                            positioner_ids=pos_id,
-                            positions=arm_chunk,
-                        )
+                    send_trajectory_pids.append(pos_id)
+
+                    positions = numpy.array(arm_chunk).astype(numpy.float64)
+                    data_pos = SendTrajectoryData.calculate_positions(positions)
+
+                    data[pos_id] = data_pos
+
+                data_send_cmd = await self.fps.send_command(
+                    "SEND_TRAJECTORY_DATA",
+                    positioner_ids=send_trajectory_pids,
+                    data=data,
+                )
+
+                if data_send_cmd.status.failed or data_send_cmd.status.timed_out:
+                    self.failed = True
+                    raise TrajectoryError(
+                        "At least one SEND_TRAJECTORY_COMMAND failed."
                     )
 
-                await asyncio.gather(*data_cmds)
-
-                for cmd in data_cmds:
-                    if cmd.status.failed or cmd.status.timed_out:
-                        self.failed = True
-                        raise TrajectoryError(
-                            "At least one SEND_TRAJECTORY_COMMAND failed."
-                        )
-
         # Finalise the trajectories
         end_traj_cmds = await self.fps.send_command(
             "TRAJECTORY_DATA_END",
@@ -400,7 +410,8 @@ class Trajectory(object):
                 "START_TRAJECTORY",
                 positioner_ids=0,
                 timeout=1,
-                n_positioners=len(self.trajectories),
+                # All positioners reply, including those not in the trajectory.
+                n_positioners=len(self.fps.positioners),
             )
 
             if command.status.failed:
@@ -414,7 +425,8 @@ class Trajectory(object):
         start_time = time.time()
 
         try:
-            assert self.move_time is not None, "move_time not set."
+            if self.move_time is None:
+                raise TrajectoryError("move_time not set.")
 
             while True:
 
@@ -468,7 +480,18 @@ class SendNewTrajectory(Command):
     broadcastable = False
     move_command = True
 
-    def __init__(self, positioner_ids, n_alpha, n_beta, **kwargs):
+    def __init__(self, positioner_ids, n_alpha=None, n_beta=None, **kwargs):
+
+        if n_alpha is not None and n_beta is not None:
+            kwargs["data"] = self.get_data(n_alpha, n_beta)
+        else:
+            assert "data" in kwargs, "n_alpha/n_beta or data need to be passed."
+
+        super().__init__(positioner_ids, **kwargs)
+
+    @staticmethod
+    def get_data(n_alpha, n_beta):
+        """Returns the data bytearray from a pair for ``(n_alpha, n_beta)``."""
 
         alpha_positions = int(n_alpha)
         beta_positions = int(n_beta)
@@ -476,9 +499,8 @@ class SendNewTrajectory(Command):
         assert alpha_positions > 0 and beta_positions > 0
 
         data = int_to_bytes(alpha_positions) + int_to_bytes(beta_positions)
-        kwargs["data"] = data
 
-        super().__init__(positioner_ids, **kwargs)
+        return data
 
 
 class SendTrajectoryData(Command):
@@ -502,22 +524,31 @@ class SendTrajectoryData(Command):
     broadcastable = False
     move_command = True
 
-    def __init__(self, positioner_ids, positions, **kwargs):
+    def __init__(self, positioner_ids, positions=None, **kwargs):
+
+        if positions is not None:
+            kwargs["data"] = self.calculate_positions(positions)
+        else:
+            assert "data" in kwargs, "positions or data need to be passed."
+
+        super().__init__(positioner_ids, **kwargs)
+
+    @staticmethod
+    def calculate_positions(positions):
+        """Converts angle-time posions to bytes data."""
 
         positions = numpy.array(positions).astype(numpy.float64)
 
         positions[:, 0] = positions[:, 0] / 360.0 * MOTOR_STEPS
         positions[:, 1] /= TIME_STEP
 
-        self.trajectory_points = positions.astype(numpy.int32)
+        positions = positions.astype(numpy.int32)
 
         data = []
-        for angle, tt in self.trajectory_points:
+        for angle, tt in positions:
             data.append(int_to_bytes(angle, dtype="i4") + int_to_bytes(tt, dtype="i4"))
 
-        kwargs["data"] = data
-
-        super().__init__(positioner_ids, **kwargs)
+        return data
 
 
 class TrajectoryDataEnd(Command):
